/*****************************************************************************
 * $Id: game_pony_debug.cc,v 1.4 2006/07/23 02:26:44 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Debug/profiling support, statistics, etc.
 *//*
 * LEGAL:	COPYRIGHT (C) 2004 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 *****************************************************************************/

#if DEBUG
#define GAME_PONY_DEBUG_CC 1
#include "game_pony_common.hh"
#include "gfx_stream.hh"

/*****************************************************************************
 * Stats constructor.
 *****************************************************************************/
GamePonyStats::GamePonyStats( void )
{
	mTracers				= 0;
	mLandQuadsDrawn			= 0;
	mLandQuadsDrawnTextured	= 0;
	mHills					= 0;
	mHillsDrawn				= 0;
	mTrees					= 0;
	mTreesDrawn				= 0;
	mLakes					= 0;
	mLakesDrawn				= 0;
}

/*****************************************************************************
 * Clear some stats prior to drawing.
 *****************************************************************************/
void
GamePonyStats::PreDrawViews( void )
{
	// Only some stats members need resetting.
	mLandQuadsDrawn			= 0;
	mLandQuadsDrawnTextured	= 0;
	mHillsDrawn				= 0;
	mTreesDrawn				= 0;
	mLakesDrawn				= 0;
}

/*****************************************************************************
 * operator<<(GamePonyStats)
 *****************************************************************************/
ostream&
operator<<( ostream& strm, const GamePonyStats& o )
{
    strm << "Statistics: \n"
         << "  land quads drawn           = " << o.mLandQuadsDrawn				<< '\n'
         << "  land quads drawn/textured  = " << o.mLandQuadsDrawnTextured		<< '\n'
         << "  hills                      = " << o.mHills						<< '\n'
         << "  hills drawn                = " << o.mHillsDrawn					<< '\n'
         << "  trees                      = " << o.mTrees						<< '\n'
         << "  trees drawn                = " << o.mTreesDrawn					<< '\n'
         << "  lakes                      = " << o.mLakes						<< '\n'
         << "  lakes drawn                = " << o.mLakesDrawn					<< '\n'
         << "[1] Count is inflated by SharedVal storing two copies (key,val)"; // << '\n'
    return strm;
}

#endif // DEBUG
