/*****************************************************************************
 * $Id: game_pony_defs.hh,v 1.11 2006/03/24 04:01:50 devel Exp $
 * @file
 * @author	Jim E. Brooks  http://www.jimbrooks.org
 * @brief	Definitions, constants, settings (some which engine depends on).
 * @remarks @verbatim
 * ----------------------------------------------------------------------------
 * This header file should only contain self-contained constants (no dependencies).
 * C++ const is preferred to use type-checking but some must be #defined
 * as some source files test them using #if.
 * NOTE: Be sure to WRITE A PERIOD IN FLOAT CONSTANTS defined by #define
 * or integer instructions could be compiled (with loss of accuracy).
 * ----------------------------------------------------------------------------
 * @endverbatim
 *//*
 * LEGAL:	COPYRIGHT (C) 2004 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE (GPL).
 *****************************************************************************/

#ifndef GAME_PONY_DEFS_HH
#define GAME_PONY_DEFS_HH 1


//------------------------------------------------------------------------------
#if COMPILING_GAME
#define GAME_MODULE_NAME_STR			"Pony"
#define GAME_MODULE_NAME_UCASE_STR		"PONY"
#define PROGRAM_EXEC_STR				"palomino" EXEC_SUFFIX_STR
#define PROGRAM_NAME_STR				"Palomino"
#define PROGRAM_LONG_NAME_UCASE_STR		"PALOMINO FLIGHT SIMULATOR"
#define PROGRAM_LONG_NAME_STR			"Palomino Flight Simulator"
#define CONFIG_FILE_APPNAME_STR			"palomino"
#define COPYRIGHT_STR					"(C) 2006"
#define COPYRIGHT_UCASE_STR				"(C) 2006"
#define AUTHORS_STR						"Jim E. Brooks"
#define AUTHORS_UCASE_STR				"JIM E. BROOKS"
#define WEBSITE_STR						"www.palomino3d.org"
#define WEBSITE_UCASE_STR				"WWW.PALOMINO3D.ORG"
#endif // COMPILING_GAME
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
#if COMPILING_GAME
const fp	EYE_JOYSTICK_ROTATE_RATIO	= 0.25;				///< prefer joystick to affect eye less than objects
#define		JOY_RESPONSE				(2.0)				///< rotation degree when joystick moved as far as possible
#define		ALTITUDE_LAND  (gGame.mAltitudeLand)			///< limited use
#define		ALTITUDE_WATER (gGame.mAltitudeWater)			///< limited use
#endif // COMPILING_GAME
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
#if COMPILING_GAME_PONY
/// TODO eliminate GROUND_LEVEL
/// GROUND_LEVEL is appropriate for most objects at ground-level.
/// By contrast, ALTITUDE_LAND has only special uses such as for land tiles.
/// The difference stems from the need for land quads to be beneath
/// ground-level objects.  See comments at game listener.
#define GROUND_LEVEL	WORLD(1.0)		///< Y coord for ground-level objects
#endif
//------------------------------------------------------------------------------


// Definitions the engine depends on.
#include "game_pony_defs_eng.hh"


#endif // GAME_PONY_DEFS_HH
